#include "ShaderConstants.fxh"
#include "util.fxh"

#include "function.fxh"

struct PS_Input
{
 float4 position: SV_Position;

 #ifndef BYPASS_PIXEL_SHADER
 lpfloat4 color: COLOR;
 snorm float2 uv0: TEXCOORD_0_FB_MSAA;
 snorm float2 uv1: TEXCOORD_1_FB_MSAA;
 #endif

 #ifdef FOG
 float4 fogColor: FOG_COLOR;
 #endif

 float4 haze: HAZE;
};

struct PS_Output
{
 float4 color: SV_Target;
};

ROOT_SIGNATURE
void main(in PS_Input PSInput, out PS_Output PSOutput) {
 #ifdef BYPASS_PIXEL_SHADER
 PSOutput.color = float4(0.0f, 0.0f, 0.0f, 0.0f);
 return;
 #else

 #if USE_TEXEL_AA
 float4 diffuse = texture2D_AA(TEXTURE_0, TextureSampler0, PSInput.uv0);
 #else
 float4 diffuse = TEXTURE_0.Sample(TextureSampler0, PSInput.uv0);
 #endif
 
	float sun = TEXTURE_1.Sample(TextureSampler1, float2(0.3, inp.uv1.y)).g;

 #ifdef SEASONS_FAR
 diffuse.a = 1.0f;
 #endif

 #if USE_ALPHA_TEST
 #ifdef ALPHA_TO_COVERAGE
 #define ALPHA_THRESHOLD 0.05
 #else
 #define ALPHA_THRESHOLD 0.5
 #endif
 if (diffuse.a < ALPHA_THRESHOLD)
  discard;
 #endif
 
 //when underwater
 bool underwater = (FOG_COLOR.b > FOG_COLOR.r && FOG_COLOR.b < 0.9 && FOG_COLOR.r < 0.3);

 #if defined(BLEND)
 diffuse.a *= PSInput.color.a;
 #endif
 
 //color of the night/shadows
 float4 dcol = TEXTURE_1.Sample(TextureSampler1, float2(inp.uv1.x, max(inp.uv1.y * 0.7, pow(inp.uv1.y, 3.0))));
 float3 dark = float3(0.82, 0.77, 0.91);
 
 dcol.rgb *= pow(dcol.rgb, 0.8 - dark);
 dcol.rgb = smoothstep(-0.04, 0.97, dcol.rgb);

 #if !defined(ALWAYS_LIT)
 diffuse *= dcol;
 #endif

 #ifndef SEASONS
 #if !USE_ALPHA_TEST && !defined(BLEND)
 diffuse.a = PSInput.color.a;
 #endif

 diffuse.rgb *= PSInput.color.rgb;
 #else
 float2 uv = PSInput.color.xy;
 diffuse.rgb *= lerp(1.0f, TEXTURE_2.Sample(TextureSampler2, uv).rgb*2.0f, PSInput.color.b);
 diffuse.rgb *= PSInput.color.aaa;
 diffuse.a = 1.0f;
 #endif
 
 //water color
 #if defined(BLEND) || defined(FOG)
 //makes water look sexy
 if (color.b > color.r) {
  diffuse.rgb *= float3(0.48, 0.97, 1.0);
 }
 #endif
 
 //monochromatic effect when raining
 #ifdef FANCY
 float3 mono = polarize3(diffuse.rgb);
 float uncolor = 1.0;
 diffuse.rgb = lerp(mono, diffuse.rgb, uncolor);
 #endif
 
 //side shadows
 if (color.g <= 0.5775) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.5787) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.58) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.5825) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.585) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.59) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.595) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.6) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.605) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.61) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.615) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.62) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.645) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.6487) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 }
 if (color.g <= 0.65) {
  diffuse.rgb *= lerp(float3(1.0), float3(0.976) + uv1.x * float3(0.05), 0.7);
 } //for some reason for loops aren't working so I had no choice but to do this monstrosity
 
 //cool torch thing
 diffuse.rgb = mix(diffuse.rgb + ((float3(0.68, 0.29, 0.03) + cos(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME) * cos(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME) * cos(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME)) * uv1.x), diffuse.rgb, sun);

 //world saturation/color
 diffuse.rgb = smoothstep(-0.077, 0.87, diffuse.rgb);
 diffuse.rgb *= float3(0.99, 0.91, 0.98);
 
 //underwater stuff
 if (underwater == true) {
  if (FOG_CONTROL.x <= 0.0) {
   //underwater lighting
   diffuse.rgb = lerp(diffuse.rgb + ((float3(0.13, 0.68, 0.92) + cos(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME) * cos(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME) * cos(TOTAL_REAL_WORLD_TIME) * sin(TOTAL_REAL_WORLD_TIME)) * uv1.x), diffuse.rgb, sun);
   
   //underwater color
   diffuse.rgb *= float3(0.53, 0.71, 0.84);
   
   //underwater effect
   /*diffuse.rgb += sin(((position.x + position.x + sin(TOTAL_REAL_WORLD_TIME + position.x)) + (position.z + position.z + sin(TOTAL_REAL_WORLD_TIME + position.z))) + TOTAL_REAL_WORLD_TIME) * 0.083 * diffuse.b;*/
  }
 }

 #ifdef FOG
 diffuse.rgb = lerp(diffuse.rgb, PSInput.fogColor.rgb, PSInput.fogColor.a);
 #endif
 diffuse.rgb = lerp(diffuse.rgb, PSInput.haze.rgb, PSInput.haze.a);

 PSOutput.color = diffuse;

 #ifdef VR_MODE
 // On Rift, the transition from 0 brightness to the lowest 8 bit value is abrupt, so clamp to
 // the lowest 8 bit value.
 PSOutput.color = max(PSOutput.color, 1 / 255.0f);
 #endif

 #endif // BYPASS_PIXEL_SHADER
}